@@ -5,6 +5,7 @@ from typing import Any, Dict, List, Optional, Union
 import attr
 from requests import exceptions
 
+from ..constants import DataGenerationMethod
 from ..exceptions import HTTPError
 from ..models import APIOperation, Status, TestResult, TestResultSet
 from ..schemas import BaseSchema
@@ -81,19 +82,28 @@ class BeforeExecution(CurrentOperationMixin, ExecutionEvent):
     relative_path: str = attr.ib()  # pragma: no mutate
     # The current level of recursion during stateful testing
     recursion_level: int = attr.ib()  # pragma: no mutate
+    # The way data will be generated
+    data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
     # An unique ID which connects events that happen during testing of the same API operation
     # It may be useful when multiple threads are involved where incoming events are not ordered
     correlation_id: str = attr.ib()  # pragma: no mutate
     thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
-    def from_operation(cls, operation: APIOperation, recursion_level: int, correlation_id: str) -> "BeforeExecution":
+    def from_operation(
+        cls,
+        operation: APIOperation,
+        recursion_level: int,
+        data_generation_method: DataGenerationMethod,
+        correlation_id: str,
+    ) -> "BeforeExecution":
         return cls(
             method=operation.method.upper(),
             path=operation.full_path,
             verbose_name=operation.verbose_name,
             relative_path=operation.path,
             recursion_level=recursion_level,
+            data_generation_method=data_generation_method,
             correlation_id=correlation_id,
         )
 
@@ -105,9 +115,13 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
     method: str = attr.ib()  # pragma: no mutate
     path: str = attr.ib()  # pragma: no mutate
     relative_path: str = attr.ib()  # pragma: no mutate
+    # Specification-specific operation name
+    verbose_name: str = attr.ib()  # pragma: no mutate
 
     # APIOperation test status - success / failure / error
     status: Status = attr.ib()  # pragma: no mutate
+    # The way data was generated
+    data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
     result: SerializedTestResult = attr.ib()  # pragma: no mutate
     # Test running time
     elapsed_time: float = attr.ib()  # pragma: no mutate
@@ -124,16 +138,19 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
         elapsed_time: float,
         hypothesis_output: List[str],
         operation: APIOperation,
+        data_generation_method: DataGenerationMethod,
         correlation_id: str,
     ) -> "AfterExecution":
         return cls(
             method=operation.method.upper(),
             path=operation.full_path,
             relative_path=operation.path,
+            verbose_name=operation.verbose_name,
             result=SerializedTestResult.from_test_result(result),
             status=status,
             elapsed_time=elapsed_time,
             hypothesis_output=hypothesis_output,
+            data_generation_method=data_generation_method,
             correlation_id=correlation_id,
         )
 
