@@ -1,4 +1,5 @@
 import ctypes
+import queue
 import threading
 import time
 from queue import Queue
@@ -11,7 +12,7 @@ from ..._hypothesis import create_test
 from ...models import CheckFunction, TestResultSet
 from ...stateful import Feedback, Stateful
 from ...targets import Target
-from ...types import RawAuth
+from ...types import RawAuth, RequestCert
 from ...utils import Ok, capture_hypothesis_output, get_requests_auth
 from .. import events
 from .core import BaseRunner, asgi_test, get_session, handle_schema_error, network_test, run_test, wsgi_test
@@ -21,6 +22,7 @@ def _run_task(
     test_template: Callable,
     tasks_queue: Queue,
     events_queue: Queue,
+    generator_done: threading.Event,
     checks: Iterable[CheckFunction],
     targets: Iterable[Target],
     settings: hypothesis.settings,
@@ -52,8 +54,15 @@ def _run_task(
             _run_tests(feedback.get_stateful_tests, recursion_level + 1)
 
     with capture_hypothesis_output():
-        while not tasks_queue.empty():
-            result, data_generation_method = tasks_queue.get()
+        while True:
+            try:
+                result, data_generation_method = tasks_queue.get(timeout=0.001)
+            except queue.Empty:
+                # The queue is empty & there will be no more tasks
+                if generator_done.is_set():
+                    break
+                # If there is a possibility for new tasks - try again
+                continue
             if isinstance(result, Ok):
                 operation = result.ok()
                 test_function = create_test(
@@ -78,6 +87,7 @@ def _run_task(
 def thread_task(
     tasks_queue: Queue,
     events_queue: Queue,
+    generator_done: threading.Event,
     checks: Iterable[CheckFunction],
     targets: Iterable[Target],
     settings: hypothesis.settings,
@@ -100,6 +110,7 @@ def thread_task(
             network_test,
             tasks_queue,
             events_queue,
+            generator_done,
             checks,
             targets,
             settings,
@@ -116,6 +127,7 @@ def thread_task(
 def wsgi_thread_task(
     tasks_queue: Queue,
     events_queue: Queue,
+    generator_done: threading.Event,
     checks: Iterable[CheckFunction],
     targets: Iterable[Target],
     settings: hypothesis.settings,
@@ -129,6 +141,7 @@ def wsgi_thread_task(
         wsgi_test,
         tasks_queue,
         events_queue,
+        generator_done,
         checks,
         targets,
         settings,
@@ -143,6 +156,7 @@ def wsgi_thread_task(
 def asgi_thread_task(
     tasks_queue: Queue,
     events_queue: Queue,
+    generator_done: threading.Event,
     checks: Iterable[CheckFunction],
     targets: Iterable[Target],
     settings: hypothesis.settings,
@@ -157,6 +171,7 @@ def asgi_thread_task(
         asgi_test,
         tasks_queue,
         events_queue,
+        generator_done,
         checks,
         targets,
         settings,
@@ -180,13 +195,36 @@ class ThreadPoolRunner(BaseRunner):
 
     workers_num: int = attr.ib(default=2)  # pragma: no mutate
     request_tls_verify: Union[bool, str] = attr.ib(default=True)  # pragma: no mutate
+    request_cert: Optional[RequestCert] = attr.ib(default=None)  # pragma: no mutate
 
-    def _execute(self, results: TestResultSet) -> Generator[events.ExecutionEvent, None, None]:
+    def _execute(
+        self, results: TestResultSet, stop_event: threading.Event
+    ) -> Generator[events.ExecutionEvent, None, None]:
         """All events come from a queue where different workers push their events."""
-        tasks_queue = self._get_tasks_queue()
+        # Instead of generating all tests at once, we do it when there is a free worker to pick it up
+        # This is extremely important for memory consumption when testing large schemas
+        # IMPLEMENTATION NOTE:
+        # It would be better to have a separate producer thread and communicate via threading events.
+        # Though it is a bit more complex, so the current solution is suboptimal in terms of resources utilization,
+        # but good enough and easy enough to implement.
+        tasks_generator = (
+            (operation, data_generation_method)
+            for operation in self.schema.get_all_operations()
+            for data_generation_method in self.schema.data_generation_methods
+        )
+        generator_done = threading.Event()
+        tasks_queue: Queue = Queue()
+        # Add at least `workers_num` tasks first, so all workers are busy
+        for _ in range(self.workers_num):
+            try:
+                # SAFETY: Workers didn't start yet, direct modification is OK
+                tasks_queue.queue.append(next(tasks_generator))
+            except StopIteration:
+                generator_done.set()
+                break
         # Events are pushed by workers via a separate queue
         events_queue: Queue = Queue()
-        workers = self._init_workers(tasks_queue, events_queue, results)
+        workers = self._init_workers(tasks_queue, events_queue, results, generator_done)
 
         def stop_workers() -> None:
             for worker in workers:
@@ -205,36 +243,35 @@ class ThreadPoolRunner(BaseRunner):
                 is_finished = all(not worker.is_alive() for worker in workers)
                 while not events_queue.empty():
                     event = events_queue.get()
-                    yield event
-                    if isinstance(event, events.Interrupted):
-                        # Thread received SIGINT
+                    if stop_event.is_set() or isinstance(event, events.Interrupted) or self._should_stop(event):
                         # We could still have events in the queue, but ignore them to keep the logic simple
                         # for now, could be improved in the future to show more info in such corner cases
-                        raise ThreadInterrupted
-        except ThreadInterrupted:
-            stop_workers()
+                        stop_workers()
+                        is_finished = True
+                        if stop_event.is_set():
+                            # Discard the event. The invariant is: the next event after `stream.stop()` is `Finished`
+                            break
+                    yield event
+                    # When we know that there are more tasks, put another task to the queue.
+                    # The worker might not actually finish the current one yet, but we put the new one now, so
+                    # the worker can immediately pick it up when the current one is done
+                    if isinstance(event, events.BeforeExecution) and not generator_done.is_set():
+                        try:
+                            tasks_queue.put(next(tasks_generator))
+                        except StopIteration:
+                            generator_done.set()
         except KeyboardInterrupt:
             stop_workers()
             yield events.Interrupted()
 
-    def _get_tasks_queue(self) -> Queue:
-        """All API operations are distributed among all workers via a queue."""
-        tasks_queue: Queue = Queue()
-        tasks_queue.queue.extend(
-            [
-                (operation, data_generation_method)
-                for operation in self.schema.get_all_operations()
-                for data_generation_method in self.schema.data_generation_methods
-            ]
-        )
-        return tasks_queue
-
-    def _init_workers(self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet) -> List[threading.Thread]:
+    def _init_workers(
+        self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet, generator_done: threading.Event
+    ) -> List[threading.Thread]:
         """Initialize & start workers that will execute tests."""
         workers = [
             threading.Thread(
                 target=self._get_task(),
-                kwargs=self._get_worker_kwargs(tasks_queue, events_queue, results),
+                kwargs=self._get_worker_kwargs(tasks_queue, events_queue, results, generator_done),
                 name=f"schemathesis_{num}",
             )
             for num in range(self.workers_num)
@@ -246,10 +283,13 @@ class ThreadPoolRunner(BaseRunner):
     def _get_task(self) -> Callable:
         return thread_task
 
-    def _get_worker_kwargs(self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet) -> Dict[str, Any]:
+    def _get_worker_kwargs(
+        self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet, generator_done: threading.Event
+    ) -> Dict[str, Any]:
         return {
             "tasks_queue": tasks_queue,
             "events_queue": events_queue,
+            "generator_done": generator_done,
             "checks": self.checks,
             "targets": self.targets,
             "settings": self.hypothesis_settings,
@@ -263,6 +303,7 @@ class ThreadPoolRunner(BaseRunner):
             "kwargs": {
                 "request_timeout": self.request_timeout,
                 "request_tls_verify": self.request_tls_verify,
+                "request_cert": self.request_cert,
                 "store_interactions": self.store_interactions,
                 "max_response_time": self.max_response_time,
                 "dry_run": self.dry_run,
@@ -274,10 +315,13 @@ class ThreadPoolWSGIRunner(ThreadPoolRunner):
     def _get_task(self) -> Callable:
         return wsgi_thread_task
 
-    def _get_worker_kwargs(self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet) -> Dict[str, Any]:
+    def _get_worker_kwargs(
+        self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet, generator_done: threading.Event
+    ) -> Dict[str, Any]:
         return {
             "tasks_queue": tasks_queue,
             "events_queue": events_queue,
+            "generator_done": generator_done,
             "checks": self.checks,
             "targets": self.targets,
             "settings": self.hypothesis_settings,
@@ -300,10 +344,13 @@ class ThreadPoolASGIRunner(ThreadPoolRunner):
     def _get_task(self) -> Callable:
         return asgi_thread_task
 
-    def _get_worker_kwargs(self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet) -> Dict[str, Any]:
+    def _get_worker_kwargs(
+        self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet, generator_done: threading.Event
+    ) -> Dict[str, Any]:
         return {
             "tasks_queue": tasks_queue,
             "events_queue": events_queue,
+            "generator_done": generator_done,
             "checks": self.checks,
             "targets": self.targets,
             "settings": self.hypothesis_settings,
@@ -318,7 +365,3 @@ class ThreadPoolASGIRunner(ThreadPoolRunner):
                 "dry_run": self.dry_run,
             },
         }
-
-
-class ThreadInterrupted(Exception):
-    """Special exception when worker thread received SIGINT."""
