@@ -2,6 +2,6 @@ import attr
 
 
 @attr.s(slots=True)
-class TestJob:
-    job_id: str = attr.ib()
+class TestRun:
+    run_id: str = attr.ib()
     short_url: str = attr.ib()
