@@ -1,5 +1,5 @@
 from functools import partial
-from typing import Any, Dict, Generator, List, Optional, Sequence, Tuple, cast
+from typing import Any, Dict, Generator, List, Optional, Sequence, Tuple, Type, TypeVar, Union, cast
 from urllib.parse import urlsplit
 
 import attr
@@ -8,6 +8,7 @@ import requests
 from hypothesis import strategies as st
 from hypothesis.strategies import SearchStrategy
 from hypothesis_graphql import strategies as gql_st
+from requests.structures import CaseInsensitiveDict
 
 from ...checks import not_a_server_error
 from ...constants import DataGenerationMethod
@@ -16,7 +17,8 @@ from ...hooks import HookDispatcher
 from ...models import APIOperation, Case, CheckFunction, OperationDefinition
 from ...schemas import BaseSchema
 from ...stateful import Stateful, StatefulTest
-from ...utils import GenericResponse, Ok, Result
+from ...types import Body, Cookies, Headers, NotSet, PathParameters, Query
+from ...utils import NOT_SET, GenericResponse, Ok, Result
 
 
 @attr.s(slots=True, repr=False)  # pragma: no mutate
@@ -26,14 +28,24 @@ class GraphQLCase(Case):
     ) -> Dict[str, Any]:
         final_headers = self._get_headers(headers)
         base_url = self._get_base_url(base_url)
-        return {"method": self.method, "url": base_url, "json": {"query": self.body}, "headers": final_headers}
+        kwargs: Dict[str, Any] = {"method": self.method, "url": base_url, "headers": final_headers}
+        # There is no direct way to have bytes here, but it is a useful pattern to support.
+        # It also unifies GraphQLCase with its Open API counterpart where bytes may come from external examples
+        if isinstance(self.body, bytes):
+            kwargs["data"] = self.body
+            # Assume that the payload is JSON, not raw GraphQL queries
+            kwargs["headers"].setdefault("Content-Type", "application/json")
+        else:
+            kwargs["json"] = {"query": self.body}
+        return kwargs
 
     def as_werkzeug_kwargs(self, headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
         final_headers = self._get_headers(headers)
         return {
             "method": self.method,
             "path": self.operation.schema.get_full_path(self.formatted_path),
-            "headers": final_headers,
+            # Convert to a regular dictionary, as we use `CaseInsensitiveDict` which is not supported by Werkzeug
+            "headers": dict(final_headers),
             "query_string": self.query,
             "json": {"query": self.body},
         }
@@ -59,6 +71,9 @@ class GraphQLCase(Case):
         return super().call_asgi(app=app, base_url=base_url, headers=headers, **kwargs)
 
 
+C = TypeVar("C", bound=Case)
+
+
 @attr.s()  # pragma: no mutate
 class GraphQLSchema(BaseSchema):
     def get_full_path(self, path: str) -> str:
@@ -81,6 +96,17 @@ class GraphQLSchema(BaseSchema):
     def _get_base_path(self) -> str:
         return cast(str, urlsplit(self.location).path)
 
+    @property
+    def operations_count(self) -> int:
+        raw_schema = self.raw_schema["__schema"]
+        if "queryType" not in raw_schema:
+            return 0
+        query_type_name = raw_schema["queryType"]["name"]
+        for type_def in raw_schema.get("types", []):
+            if type_def["name"] == query_type_name:
+                return len(type_def["fields"])
+        return 0
+
     def get_all_operations(self) -> Generator[Result[APIOperation, InvalidSchema], None, None]:
         schema = self.client_schema
         if schema.query_type is None:
@@ -106,7 +132,7 @@ class GraphQLSchema(BaseSchema):
         hooks: Optional[HookDispatcher] = None,
         data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
     ) -> SearchStrategy:
-        constructor = partial(GraphQLCase, operation=operation)
+        constructor = partial(GraphQLCase, operation=operation, data_generation_method=data_generation_method)
         return st.builds(constructor, body=gql_st.query(self.client_schema, fields=[operation.verbose_name]))
 
     def get_strategies_from_examples(self, operation: APIOperation) -> List[SearchStrategy[Case]]:
@@ -116,3 +142,25 @@ class GraphQLSchema(BaseSchema):
         self, response: GenericResponse, operation: APIOperation, stateful: Optional[Stateful]
     ) -> Sequence[StatefulTest]:
         return []
+
+    def make_case(
+        self,
+        *,
+        case_cls: Type[C],
+        operation: APIOperation,
+        path_parameters: Optional[PathParameters] = None,
+        headers: Optional[Headers] = None,
+        cookies: Optional[Cookies] = None,
+        query: Optional[Query] = None,
+        body: Union[Body, NotSet] = NOT_SET,
+        media_type: Optional[str] = None,
+    ) -> C:
+        return case_cls(
+            operation=operation,
+            path_parameters=path_parameters,
+            headers=CaseInsensitiveDict(headers) if headers is not None else headers,
+            cookies=cookies,
+            query=query,
+            body=body,
+            media_type=media_type,
+        )
