@@ -4,18 +4,37 @@ from collections import defaultdict
 from contextlib import ExitStack, contextmanager
 from copy import deepcopy
 from difflib import get_close_matches
+from hashlib import sha1
 from json import JSONDecodeError
-from typing import Any, Callable, ClassVar, Dict, Generator, Iterable, List, Optional, Sequence, Tuple, Type, Union
+from threading import RLock
+from typing import (
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    Generator,
+    Iterable,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 from urllib.parse import urlsplit
 
+import attr
 import jsonschema
 import requests
 from hypothesis.strategies import SearchStrategy
+from requests.structures import CaseInsensitiveDict
 
 from ... import failures
-from ...constants import DataGenerationMethod
+from ...constants import HTTP_METHODS, DataGenerationMethod
 from ...exceptions import (
     InvalidSchema,
+    UsageError,
     get_missing_content_type_error,
     get_response_parsing_error,
     get_schema_validation_error,
@@ -24,11 +43,20 @@ from ...hooks import HookContext, HookDispatcher
 from ...models import APIOperation, Case, OperationDefinition
 from ...schemas import BaseSchema
 from ...stateful import APIStateMachine, Stateful, StatefulTest
-from ...types import FormData
-from ...utils import Err, GenericResponse, Ok, Result, get_response_payload, is_json_media_type
+from ...types import Body, Cookies, FormData, Headers, NotSet, PathParameters, Query
+from ...utils import (
+    NOT_SET,
+    Err,
+    GenericResponse,
+    Ok,
+    Result,
+    get_response_payload,
+    is_json_media_type,
+    traverse_schema,
+)
 from . import links, serialization
 from ._hypothesis import get_case_strategy
-from .converter import to_json_schema_recursive
+from .converter import to_json_schema, to_json_schema_recursive
 from .examples import get_strategies_from_examples
 from .filters import (
     should_skip_by_operation_id,
@@ -53,14 +81,21 @@ SCHEMA_ERROR_MESSAGE = "Schema parsing failed. Please check your schema."
 SCHEMA_PARSING_ERRORS = (KeyError, AttributeError, jsonschema.exceptions.RefResolutionError)
 
 
+@attr.s(eq=False, repr=False)
 class BaseOpenAPISchema(BaseSchema):
     nullable_name: str
     links_field: str
-    allowed_http_methods: Tuple[str, ...]
     security: BaseSecurityProcessor
-    parameter_cls: Type[OpenAPIParameter]
-    component_locations: ClassVar[Tuple[str, ...]] = ()
+    component_locations: ClassVar[Tuple[Tuple[str, ...], ...]] = ()
     _operations_by_id: Dict[str, APIOperation]
+    _inline_reference_cache: Dict[str, Any]
+    # Inline references cache can be populated from multiple threads, therefore we need some synchronisation to avoid
+    # excessive resolving
+    _inline_reference_cache_lock: RLock
+
+    def __attrs_post_init__(self) -> None:
+        self._inline_reference_cache = {}
+        self._inline_reference_cache_lock = RLock()
 
     @property  # pragma: no mutate
     def spec_version(self) -> str:
@@ -77,6 +112,42 @@ class BaseOpenAPISchema(BaseSchema):
         info = self.raw_schema["info"]
         return f"{self.__class__.__name__} for {info['title']} ({info['version']})"
 
+    def _should_skip(self, method: str, definition: Dict[str, Any]) -> bool:
+        return (
+            method not in HTTP_METHODS
+            or should_skip_method(method, self.method)
+            or should_skip_deprecated(definition.get("deprecated", False), self.skip_deprecated_operations)
+            or should_skip_by_tag(definition.get("tags"), self.tag)
+            or should_skip_by_operation_id(definition.get("operationId"), self.operation_id)
+        )
+
+    @property
+    def operations_count(self) -> int:
+        try:
+            paths = self.raw_schema["paths"]
+        except KeyError:
+            return 0
+        total = 0
+        resolve = self.resolver.resolve
+        for path, methods in paths.items():
+            full_path = self.get_full_path(path)
+            if should_skip_endpoint(full_path, self.endpoint):
+                continue
+            try:
+                if "$ref" in methods:
+                    _, resolved_methods = resolve(methods["$ref"])
+                else:
+                    resolved_methods = methods
+                # Straightforward iteration is faster than converting to a set & calculating length.
+                for method, definition in resolved_methods.items():
+                    if self._should_skip(method, definition):
+                        continue
+                    total += 1
+            except SCHEMA_PARSING_ERRORS:
+                # Ignore errors
+                continue
+        return total
+
     def get_all_operations(self) -> Generator[Result[APIOperation, InvalidSchema], None, None]:
         """Iterate over all operations defined in the API.
 
@@ -113,18 +184,13 @@ class BaseOpenAPISchema(BaseSchema):
                     try:
                         # Setting a low recursion limit doesn't solve the problem with recursive references & inlining
                         # too much but decreases the number of cases when Schemathesis stuck on this step.
-                        with self.resolver.in_scope(scope):
+                        self.resolver.push_scope(scope)
+                        try:
                             resolved_definition = self.resolver.resolve_all(definition, RECURSION_DEPTH_LIMIT - 5)
+                        finally:
+                            self.resolver.pop_scope()
                         # Only method definitions are parsed
-                        if (
-                            method not in self.allowed_http_methods
-                            or should_skip_method(method, self.method)
-                            or should_skip_deprecated(
-                                resolved_definition.get("deprecated", False), self.skip_deprecated_operations
-                            )
-                            or should_skip_by_tag(resolved_definition.get("tags"), self.tag)
-                            or should_skip_by_operation_id(resolved_definition.get("operationId"), self.operation_id)
-                        ):
+                        if self._should_skip(method, resolved_definition):
                             continue
                         parameters = self.collect_parameters(
                             itertools.chain(resolved_definition.get("parameters", ()), common_parameters),
@@ -186,6 +252,7 @@ class BaseOpenAPISchema(BaseSchema):
         for parameter in parameters:
             operation.add_parameter(parameter)
         self.security.process_definitions(self.raw_schema, operation, self.resolver)
+        self.dispatch_hook("before_init_operation", HookContext(operation=operation), operation)
         return operation
 
     @property
@@ -218,10 +285,13 @@ class BaseOpenAPISchema(BaseSchema):
             scope, raw_methods = self._resolve_methods(methods)
             common_parameters = self.resolver.resolve_all(methods.get("parameters", []), RECURSION_DEPTH_LIMIT - 5)
             for method, definition in methods.items():
-                if method not in self.allowed_http_methods or "operationId" not in definition:
+                if method not in HTTP_METHODS or "operationId" not in definition:
                     continue
-                with self.resolver.in_scope(scope):
+                self.resolver.push_scope(scope)
+                try:
                     resolved_definition = self.resolver.resolve_all(definition, RECURSION_DEPTH_LIMIT - 5)
+                finally:
+                    self.resolver.pop_scope()
                 parameters = self.collect_parameters(
                     itertools.chain(resolved_definition.get("parameters", ()), common_parameters), resolved_definition
                 )
@@ -404,7 +474,9 @@ class BaseOpenAPISchema(BaseSchema):
                     validation_message=exc.msg, document=exc.doc, position=exc.pos, lineno=exc.lineno, colno=exc.colno
                 ),
             ) from exc
-        resolver = ConvertingResolver(self.location or "", self.raw_schema, nullable_name=self.nullable_name)
+        resolver = ConvertingResolver(
+            self.location or "", self.raw_schema, nullable_name=self.nullable_name, is_response_schema=True
+        )
         with in_scopes(resolver, scopes):
             try:
                 jsonschema.validate(data, schema, cls=jsonschema.Draft4Validator, resolver=resolver)
@@ -422,18 +494,91 @@ class BaseOpenAPISchema(BaseSchema):
                 ) from exc
         return None  # explicitly return None for mypy
 
+    @property
+    def rewritten_components(self) -> Dict[str, Any]:
+        if not hasattr(self, "_rewritten_components"):
+
+            def callback(_schema: Dict[str, Any], nullable_name: str) -> Dict[str, Any]:
+                _schema = to_json_schema(_schema, nullable_name=nullable_name, copy=False)
+                return self._rewrite_references(_schema, self.resolver)
+
+            # pylint: disable=attribute-defined-outside-init
+            # Different spec versions allow different keywords to store possible reference targets
+            components: Dict[str, Any] = {}
+            for path in self.component_locations:
+                schema = self.raw_schema
+                target = components
+                for chunk in path:
+                    if chunk in schema:
+                        schema = schema[chunk]
+                        target = target.setdefault(chunk, {})
+                    else:
+                        break
+                else:
+                    target.update(traverse_schema(deepcopy(schema), callback, self.nullable_name))
+            self._rewritten_components = components
+        return self._rewritten_components
+
     def prepare_schema(self, schema: Any) -> Any:
         """Inline Open API definitions.
 
         Inlining components helps `hypothesis-jsonschema` generate data that involves non-resolved references.
         """
         schema = deepcopy(schema)
-        # Different spec versions allow different keywords to store possible reference targets
-        for key in self.component_locations:
-            if key in self.raw_schema:
-                schema[key] = to_json_schema_recursive(self.raw_schema[key], self.nullable_name)
+        schema = traverse_schema(schema, self._rewrite_references, self.resolver)
+        schema.update(self.rewritten_components)
+        # If there are any cached references - add them to the resulting schema.
+        # Note that not all of them might be used for data generation, but at this point it is the simplest way to go
+        if self._inline_reference_cache:
+            schema[INLINED_REFERENCES_KEY] = self._inline_reference_cache
         return schema
 
+    def _rewrite_references(self, schema: Dict[str, Any], resolver: InliningResolver) -> Dict[str, Any]:
+        """Rewrite references present in the schema.
+
+        The idea is to resolve references, cache the result and replace these references with new ones
+        that point to a local path which is populated from this cache later on.
+        """
+        reference = schema.get("$ref")
+        # If `$ref` is not a property name and should be processed
+        if reference is not None and isinstance(reference, str) and not reference.startswith("#/"):
+            key = _make_reference_key(resolver._scopes_stack, reference)
+            with self._inline_reference_cache_lock:
+                if key not in self._inline_reference_cache:
+                    with resolver.resolving(reference) as resolved:
+                        # Resolved object also may have references
+                        self._inline_reference_cache[key] = traverse_schema(
+                            resolved, lambda s: self._rewrite_references(s, resolver)
+                        )
+            # Rewrite the reference with the new location
+            schema["$ref"] = f"#/{INLINED_REFERENCES_KEY}/{key}"
+        return schema
+
+
+def _make_reference_key(scopes: List[str], reference: str) -> str:
+    """A name under which the resolved reference data will be stored."""
+    # Using a hexdigest is the simplest way to associate practically unique keys with each reference
+    digest = sha1()
+    for scope in scopes:
+        digest.update(scope.encode("utf-8"))
+        # Separator to avoid collissions like this: ["a"], "bc" vs. ["ab"], "c". Otherwise, the resulting digest
+        # will be the same for both cases
+        digest.update(b"#")
+    digest.update(reference.encode("utf-8"))
+    return digest.hexdigest()
+
+
+INLINED_REFERENCES_KEY = "x-inlined"
+
+
+@contextmanager
+def in_scope(resolver: jsonschema.RefResolver, scope: str) -> Generator[None, None, None]:
+    resolver.push_scope(scope)
+    try:
+        yield
+    finally:
+        resolver.pop_scope()
+
 
 @contextmanager
 def in_scopes(resolver: jsonschema.RefResolver, scopes: List[str]) -> Generator[None, None, None]:
@@ -445,22 +590,21 @@ def in_scopes(resolver: jsonschema.RefResolver, scopes: List[str]) -> Generator[
     """
     with ExitStack() as stack:
         for scope in scopes:
-            stack.enter_context(resolver.in_scope(scope))
+            stack.enter_context(in_scope(resolver, scope))
         yield
 
 
 OPENAPI_20_DEFAULT_BODY_MEDIA_TYPE = "application/json"
 OPENAPI_20_DEFAULT_FORM_MEDIA_TYPE = "multipart/form-data"
+C = TypeVar("C", bound=Case)
 
 
 class SwaggerV20(BaseOpenAPISchema):
     nullable_name = "x-nullable"
     example_field = "x-example"
     examples_field = "x-examples"
-    allowed_http_methods: Tuple[str, ...] = ("get", "put", "post", "delete", "options", "head", "patch")
-    parameter_cls: Type[OpenAPIParameter] = OpenAPI20Parameter
     security = SwaggerSecurityProcessor()
-    component_locations: ClassVar[Tuple[str, ...]] = ("definitions", "parameters", "responses")
+    component_locations: ClassVar[Tuple[Tuple[str, ...], ...]] = (("definitions",),)
     links_field = "x-links"
 
     @property
@@ -523,7 +667,7 @@ class SwaggerV20(BaseOpenAPISchema):
             return scopes, None
         # Extra conversion to JSON Schema is needed here if there was one $ref in the input
         # because it is not converted
-        return scopes, to_json_schema_recursive(schema, self.nullable_name)
+        return scopes, to_json_schema_recursive(schema, self.nullable_name, is_response_schema=True)
 
     def get_content_types(self, operation: APIOperation, response: GenericResponse) -> List[str]:
         produces = operation.definition.raw.get("produces", None)
@@ -573,6 +717,41 @@ class SwaggerV20(BaseOpenAPISchema):
     def get_request_payload_content_types(self, operation: APIOperation) -> List[str]:
         return self._get_consumes_for_operation(operation.definition.resolved)
 
+    def make_case(
+        self,
+        *,
+        case_cls: Type[C],
+        operation: APIOperation,
+        path_parameters: Optional[PathParameters] = None,
+        headers: Optional[Headers] = None,
+        cookies: Optional[Cookies] = None,
+        query: Optional[Query] = None,
+        body: Union[Body, NotSet] = NOT_SET,
+        media_type: Optional[str] = None,
+    ) -> C:
+        if body is not NOT_SET and media_type is None:
+            # If the user wants to send payload, then there should be a media type, otherwise the payload is ignored
+            media_types = operation.get_request_payload_content_types()
+            if len(media_types) == 1:
+                # The only available option
+                media_type = media_types[0]
+            else:
+                media_types_repr = ", ".join(media_types)
+                raise UsageError(
+                    "Can not detect appropriate media type. "
+                    "You can either specify one of the defined media types "
+                    f"or pass any other media type available for serialization. Defined media types: {media_types_repr}"
+                )
+        return case_cls(
+            operation=operation,
+            path_parameters=path_parameters,
+            headers=CaseInsensitiveDict(headers) if headers is not None else headers,
+            cookies=cookies,
+            query=query,
+            body=body,
+            media_type=media_type,
+        )
+
     def _get_consumes_for_operation(self, definition: Dict[str, Any]) -> List[str]:
         """Get the `consumes` value for the given API operation.
 
@@ -591,10 +770,8 @@ class OpenApi30(SwaggerV20):  # pylint: disable=too-many-ancestors
     nullable_name = "nullable"
     example_field = "example"
     examples_field = "examples"
-    allowed_http_methods = SwaggerV20.allowed_http_methods + ("trace",)
     security = OpenAPISecurityProcessor()
-    parameter_cls = OpenAPI30Parameter
-    component_locations = ("components",)
+    component_locations = (("components", "schemas"),)
     links_field = "links"
 
     @property
@@ -621,8 +798,11 @@ class OpenApi30(SwaggerV20):  # pylint: disable=too-many-ancestors
         collected: List[OpenAPIParameter] = [OpenAPI30Parameter(definition=parameter) for parameter in parameters]
         if "requestBody" in definition:
             required = definition["requestBody"].get("required", False)
+            description = definition["requestBody"].get("description")
             for media_type, content in definition["requestBody"]["content"].items():
-                collected.append(OpenAPI30Body(content, media_type=media_type, required=required))
+                collected.append(
+                    OpenAPI30Body(content, description=description, media_type=media_type, required=required)
+                )
         return collected
 
     def get_response_schema(self, definition: Dict[str, Any], scope: str) -> Tuple[List[str], Optional[Dict[str, Any]]]:
@@ -633,7 +813,7 @@ class OpenApi30(SwaggerV20):  # pylint: disable=too-many-ancestors
         if option and "schema" in option:
             # Extra conversion to JSON Schema is needed here if there was one $ref in the input
             # because it is not converted
-            return scopes, to_json_schema_recursive(option["schema"], self.nullable_name)
+            return scopes, to_json_schema_recursive(option["schema"], self.nullable_name, is_response_schema=True)
         return scopes, None
 
     def get_strategies_from_examples(self, operation: APIOperation) -> List[SearchStrategy[Case]]:
